----------------------------
-- Test 180: Capture Zone --
----------------------------
--
-- RedFor and BluFor are fighting over territory "ArmourBattleZone"
-- Two warehouses, one of each coalition, will send troops to capture the "ArmourBattleZone"
-- 
-- New forces are deployed when:
-- * The other coalition captures the zone,
-- * a group is heavily damaged or out of ammo.
-- 
-- This leads to a long lasting battle for the zone as both warehouses have a large supply of ground forces.
-- 
-- Other classes used here are
-- * ZONE_CAPTURE_COALITION to monitor the owning coalition of the battle zone and
-- * SUPPRESSION to make units retreat when they are out of ammo or damaged badly.
-- 
-- Tip: Use time speed up LCRTL+Z or stuff takes ages. 

-- Display mission time every 30 seconds.
-- SCHEDULER:New(nil, UTILS.DisplayMissionTime, {5}, 30, 30)

----------------------------------------------

-- The battle zone.
local BattleZone=ZONE:New("Objective A")
local BattleZoneName=BattleZone:GetName()

-- Warehouse table.
local warehouse={}

-- Blue warehouses
warehouse.US = WAREHOUSE:New(STATIC:FindByName("WarehouseBlue1"), "Blue") --Functional.Warehouse#WAREHOUSE
warehouse.Ru = WAREHOUSE:New(STATIC:FindByName("WarehouseRed1"), "Red")  --Functional.Warehouse#WAREHOUSE

-- Define spawn zones.
warehouse.US:SetSpawnZone(ZONE:New("WarehouseBlueSpawnZone1"))
warehouse.Ru:SetSpawnZone(ZONE:New("WarehouseRedSpawnZone1"))

-- Capture zone.
local ZoneCaptureCoalition = ZONE_CAPTURE_COALITION:New(BattleZone, coalition.side.NEUTRAL) 
ZoneCaptureCoalition:SetMonitorHits(true)

----------------------------------------------

-- Start warehouses.
warehouse.US:Start()
warehouse.Ru:Start()

-- Add assets to the warehouses.
warehouse.US:AddAsset("BlueTankGroup", 100)
warehouse.Ru:AddAsset("RedTankGroup", 100)

-- Self request of RedFor troops to initially capture the battle zone.
warehouse.Ru:AddRequest(warehouse.Ru, WAREHOUSE.Descriptor.ATTRIBUTE, WAREHOUSE.Attribute.GROUND_TANK, 1, nil, nil, nil, BattleZoneName)


-- Take care of the spawned units.
function warehouse.US:OnAfterSelfRequest(From, Event, To, Groupset, Request)
  local groupset=Groupset --Core.Set#SET_GROUP
  local request=Request   --Functional.Warehouse#WAREHOUSE.Pendingitem
  
  -- Get assignment of this request.
  local assignment=warehouse.US:GetAssignment(request)
  
  if assignment==BattleZoneName then
    
    for _,_group in pairs(groupset:GetSet()) do
      local group=_group --Wrapper.Group#GROUP
      
      -- Route group to Battle zone.
      local ToCoord=BattleZone:GetRandomCoordinate()
      group:RouteGroundOnRoad(ToCoord, group:GetSpeedMax()*0.8, nil, "Custom")
      --group:PatrolZones({BattleZone}, group:GetSpeedMax()*0.8, "Custom", 5, 50)
      
      -- Set up suppression.
      local suppression=SUPPRESSION:New(group)
      
      -- Set retreat zone, which is the same as the warehouse spawn zone here.
      suppression:SetRetreatZone(ZONE:FindByName("WarehouseBlueSpawnZone1"))
      
      --- Function called when retreat is ordered.
      function suppression:OnAfterRetreat(Controllable,From,Event,To)

        local text=string.format("Group %s is retreating to base.", tostring(group:GetName()))

        -- Check if zone is still in control of Russian troops.        
        if ZoneCaptureCoalition:GetCoalition()==coalition.side.RED then
      
          -- Get number of assets.
          local n=warehouse.US:GetNumberOfAssets(WAREHOUSE.Descriptor.ATTRIBUTE, WAREHOUSE.Attribute.GROUND_TANK)
          
          -- Request new assets to replace the retreated ones.
          if n>0 then
            warehouse.US:AddRequest(warehouse.US, WAREHOUSE.Descriptor.ATTRIBUTE, WAREHOUSE.Attribute.GROUND_TANK, 1, nil, nil, nil, BattleZoneName)
            text=text.." We are sending reinforcements"
          else
            text=text.." No reinforcements available."
          end
          
        end
        
        env.info(text)
        MESSAGE:New(text, 60):ToAll()
        
      end
      
      --- Function called when troops have arrived at the "retreat zone".
      function suppression:OnAfterRetreated(Controllable, From, Event, To)
        local text=string.format("Group %s retreated and will join next attack", tostring(group:GetName()))
        env.info(text)
        MESSAGE:New(text, 60):ToAll()
        warehouse.US:AddAsset(group, 1)
      end
      
      -- Start suppression.
      suppression:Start()
      
    end
        
  end
  
end

-- Take care of the spawned units.
function warehouse.Ru:OnAfterSelfRequest(From, Event, To, Groupset, Request)
  local groupset=Groupset --Core.Set#SET_GROUP
  local request=Request   --Functional.Warehouse#WAREHOUSE.Pendingitem
  
  -- Get assignment of this request.
  local assignment=warehouse.Ru:GetAssignment(request)
  
  if assignment==BattleZoneName then
    
    for _,_group in pairs(groupset:GetSet()) do
      local group=_group --Wrapper.Group#GROUP
      
      -- Route group to Battle zone.
      local ToCoord=BattleZone:GetRandomCoordinate()
      group:RouteGroundTo(ToCoord, group:GetSpeedMax()*0.8, "Custom")
      
      --group:PatrolZones({BattleZone}, group:GetSpeedMax()*0.8, "Custom", 5, 50)
      
      -- Set up suppression.
      local suppression=SUPPRESSION:New(group)
      
      -- Set retreat zone, which is the same as the warehouse spawn zone here.
      suppression:SetRetreatZone(ZONE:FindByName("WarehouseRedSpawnZone1"))
      
      --- Function called when retreat is ordered.
      function suppression:OnAfterRetreat(Controllable,From,Event,To)

        -- Message text
        local text=string.format("Group %s is retreating to base.", tostring(group:GetName()))

        -- Check if zone is still in control of US troops.        
        if ZoneCaptureCoalition:GetCoalition()==coalition.side.BLUE then
      
          -- Get number of assets.
          local n=warehouse.Ru:GetNumberOfAssets(WAREHOUSE.Descriptor.ATTRIBUTE, WAREHOUSE.Attribute.GROUND_TANK)
          
          -- Request new assets to replace the retreated ones.
          if n>0 then
            warehouse.Ru:AddRequest(warehouse.Ru, WAREHOUSE.Descriptor.ATTRIBUTE, WAREHOUSE.Attribute.GROUND_TANK, 1, nil, nil, nil, BattleZoneName)
            text=text.." We are sending reinforcements"
          else
            text=text.." No reinforcements available."                 
          end
                    
        end
        
        -- Info message.      
        env.info(text)
        MESSAGE:New(text, 60):ToAll()          
        
      end
      
      --- Function called when troops have arrived at the "retreat zone".
      function suppression:OnAfterRetreated(Controllable, From, Event, To)
        local text=string.format("Group %s retreated and will join next attack", tostring(group:GetName()))
        env.info(text)
        MESSAGE:New(text, 60):ToAll()
        warehouse.Ru:AddAsset(group, 1)
      end
      
      -- Start suppression.
      suppression:Start()
      
    end
        
  end
  
end

----------------------------------------------

--[[
BASE:TraceOn()
BASE:TraceClass("ZONE_CAPTURE_COALITION")
BASE:TraceLevel(1)
]]


--- Function called when only one coalition is inside the battle zone.
function ZoneCaptureCoalition:OnEnterGuarded(From, Event, To)
  if From ~= To then
    local Coalition = self:GetCoalition()
    
    local text=""
    if Coalition == coalition.side.BLUE then
      ZoneCaptureCoalition:Smoke( SMOKECOLOR.Blue )
      text=string.format( "%s is under protection of the BluFor.", ZoneCaptureCoalition:GetZoneName())
    elseif Coalition == coalition.side.RED then
      ZoneCaptureCoalition:Smoke( SMOKECOLOR.Red )
      text=string.format( "%s is under protection of RedFor.", ZoneCaptureCoalition:GetZoneName() )
    else
      text=string.format( "%s is NEUTRAL", ZoneCaptureCoalition:GetZoneName() )
    end
    
    if text~="" then
      env.info(text)
      MESSAGE:New(text, 60):ToAll()
    end
  end
end

--- Function called when the battle zone becomes empty.
function ZoneCaptureCoalition:OnEnterEmpty()
  local text=string.format( "%s is unprotected, and can be captured.", ZoneCaptureCoalition:GetZoneName() )
  env.info(text)
  MESSAGE:New(text, 60):ToAll()
end


function ZoneCaptureCoalition:OnEnterAttacked()
  local Coalition = self:GetCoalition()
  local text=""
  if Coalition == coalition.side.BLUE then
    text=string.format( "%s is under attack by RedFor", ZoneCaptureCoalition:GetZoneName() )
  else
    text=string.format( "%s is under attack by BluFor", ZoneCaptureCoalition:GetZoneName() )
  end
  
  if text~="" then
    env.info(text)
    MESSAGE:New(text, 60):ToAll()
  end
end

--- Enter captured state.
function ZoneCaptureCoalition:OnEnterCaptured()

  -- Get new owning coalition.
  local Coalition = self:GetCoalition()
  
  -- Info message depending on who is the new owner.
  local text=""
  if Coalition == coalition.side.BLUE then
    text=string.format( "%s was captured by BluFor.", ZoneCaptureCoalition:GetZoneName() )
    warehouse.Ru:AddRequest(warehouse.Ru, WAREHOUSE.Descriptor.ATTRIBUTE, WAREHOUSE.Attribute.GROUND_TANK, 2, nil, nil, nil, BattleZoneName)
  elseif Coalition == coalition.side.RED then
    text=string.format( "%s was captured by RedFor.", ZoneCaptureCoalition:GetZoneName() )
    warehouse.US:AddRequest(warehouse.US, WAREHOUSE.Descriptor.ATTRIBUTE, WAREHOUSE.Attribute.GROUND_TANK, 2, nil, nil, nil, BattleZoneName)
  else
    text=string.format( "%s was captured by NEUTRAL forces!", ZoneCaptureCoalition:GetZoneName() )
  end
  
  if text~="" then
    env.info(text)
    MESSAGE:New(text, 60):ToAll()
  end
end

-- Start capture zone script.
ZoneCaptureCoalition:Start(10, 30)
